/* ========================================================================== */
/*                                                                            */
/*   Com_Volets_roulants.c       12C508A CompilateurCC5X                      */
/*   (c) Alain Denis                                                          */
/*       06/04/2007                                                           */
/*        17/10/2012 Compilateur MikroC PIC 12F675                                      */
/*   Description                                                              */
/*    But ; Commander automatiquement louverture ou la fermeture des volets
 roulants en fonction de la luminosit.
      Un interrupteur permettra de neutraliser la marche automatique.*/
/*La commande manuelle actuelle sera conserve.

Cahier des charges

Le boitier accueillera la puissance, les raccordements des boutons poussoirs
 actuels, le raccordement du moteur denroulement du volet, le raccordement 
 du capteur de pnombre(photorsistance).
Linterrupteur manuel/auto sera accessible sans ouvrir le boitier.
A la mise sous tension, le systme devra se positionner automatiquement.
La commande actuelle sera oprationnelle lectronique sous tension ou pas, 
en  auto  comme en  manuel l, par contre il y a scurit entre les 
mouvements  monte descente 

1)        Initialisation des variables  M_Fer = 0 ; M_Ouv ; Auto = 0 ; Lum = 0
          Acquisition du Temps de tempo (longueur volet)
2)        Acquisition de  auto  manuel  Auto = Port dentre
3)        Test de la position  auto  manuel 
4)        Si  Auto =0   retour en 1
5)        Si  Auto = 1  acquisition de la Pnombre  Penombre = AN2 
6)        Test de la Penombre
7)        Si  Nuit= 1(Pnombre leve)  S.P. Fermeture volets
8)        Si  Nuit = 0 (lPenombre faible)  S.P. ouverture des volets
9)        Retour en 2
a.        SP Ouvertures volets
b.        Si M_Ouv = 0 ; C_Ouv =1 ; tempo  ; C_Ouv = 0 ; M_Ouv =1 ; M_Fer = 0 ;
c.        Retour en 9
d.        SP Ouvertures volets
e.        Si M_Fer = 0 ; C_FER =1 ; tempo ; C_Fer = 0 ; M_Fer =1 ; M_Ouv = 0 ;
f.        Retour en 9
          Interruption passage en Manu/auto                                   */
/* ========================================================================== */
/*--------Evolution du programme----------------------------------------------*/
/* 20c.c Le 01/11/2012 Ajout condition Auto dans Delay10 arrt Tempo en Manuel */
/* 20c.c Le 01/11/2012 Mis un Temps de rebondissement de contact dans Interrup */
/* 18c.c le 01/11/2012 Ajout de l'interruption Manu/Auto */
/* 17c.c Le 16/10/2012 Trouv anomalie Pullup mettre WPU aprs TRISIO */
/* 17c.c le 15/10/2012 Ajout tempo de 10s (attente stabilisation mes.de luminosit)*/
/* 17c.c le 15/09/2012 Ajout de procdure asm calibration oscillateur int. */
/* 16c.c Le 10/09/2012 commutateur pour les 4 minuteries */
/* 15c.c Le 09/09/2012 Dtection de la Nuit avec A/D et sortie Lum sur GP5 */
/* Le01/09/2012 Migration vers 12F675 avec mikroC */
/* Le 6/04/2007 Migration vers 12C508A                                        */
/*Le 27/03/2007 Fonctionnement correct16F84A----------------------------------*/

/*------------------- dclaration dans ICprog des valeurs---------------------*/

/*Oscillateur XT et tous les fusibles decoches sauf PWRT*/
     
/*--------Dclaration de fichiers externes------------------------------------*/



/*------------Dclaration des fonctions---------------------------------------*/

void interrupt (void);
void Cal_Tempo (void);
void Delay10( unsigned long int n); //Fonction delay 10ms, n Nbr de fois 10ms
void SP_Ouverture (void); //Sous programme Ouverture Volets.
void SP_Fermeture (void); // Sous programme Fermeture Volets.
void CAN (void); // dclare fonction de conversion qui renvoie une valeur int (10 bits)
void main (void);

/*------------Dclaration des variables---------------------------------------*/
unsigned int Penombre;
unsigned long  Temps, j;
bit M_Fer, M_Ouv, Nuit, M;

/*------------Dclaration des E/S --------------------------------------------*/
char pins at GPIO;// Pour mettre PORTA  zero
sbit Auto at GP3_bit; // Entre du commutateur "auto - manu"."1" en auto.
sbit b1T at GP5_bit; // bit1 de la tempo
sbit b0T at GP4_bit; // bit0 de la tempo
sbit C_Ouv at GP0_bit;// Commande relais d'Ouverture
sbit C_Fer at GP1_bit;// Commande relais de Fermeture

/*--------------------------Interruption -------------------------------------*/

void interrupt()
 {
  if (INTCON.GPIF == 1)  //Changement d'tat Auto.
  {
   for(j=0;j<1000;j++)      //Attente rebondissement de contact
    if (Auto ==0)
    {
     C_Ouv = 0 ; C_Fer = 0;
    }
    if (Auto ==1)
    {
     M_Ouv = 0 ; M_Fer = 0;
     } 
  INTCON.GPIF = 0  ;
  }
  }

  
 

/************* Programme principal ********************************************/


void main(void)
{
 // Procedure de calibration de L'oscillateur Interne
asm  {
bsf   STATUS, RP0 ;Bank 1
call  0x3FF        ;Get the cal value
movwf OSCCAL      ;Calibrate
bcf STATUS, RP0   ;Bank 0
 }


ANSEL = 0b01010100;//FOSC/16, GP2 en analogique
ADCON0 = 0b10001001;//Justifi  droite,Ref int.,GP2,..,A/N on,
CMCON = 0x07;

INTCON =0;
OPTION_REG = 0b00000111;

pins = 0; 
TRISIO = 0b00111100;// GP0-GP1 en sortie, GP2-GP5 en entre
WPU = 0b00110000; // GP4 et GP5 avec R pullup
M_Fer = 0;
M_Ouv = 0;
C_Ouv = 0;
C_Fer = 0;
Temps = 1000; // tempo de 10 secondes pour avoir une mesure pnombre stable
Delay10(Temps);
Cal_Tempo ();  //En fonction de la longueur du volet

Nuit = 0;
IOC.IOC3 = 1;    // Autorisation Interrup. sur changement tat GP3
INTCON.GPIE = 1;      // Autorisation Interruption
INTCON.GIE = 1;

/* Boucle principale*/
for (;;) {
while ( Auto == 0 );// Tant que l'on n'est pas en Auto le programme reste l.
          {       
          CAN ();   // acquisition de la Penombre AN2
if (Penombre < 470)     // Soit 2.3 volts en entre
    {
   Nuit = 0;
    }
     if (Penombre > 665)   // Soit 3,25 volts en entre
             {
              Nuit = 1;
              }
     if (Nuit == 0)// Luminost est claire vers Sous-programme Ouverture.
                    {
                     SP_Ouverture();
                     }
                     else // Luminost est sombre vers Sous-programme Fermeture.
                     {
                     SP_Fermeture();
                     }
               }
 }
}
/* *******************Sous-Programme Ouverture******************************/
void SP_Ouverture (void)
{
if (M_Ouv == 0)
  {
  C_Ouv = 1;
  Delay10(Temps);// Commande d'ouverture
  C_Ouv = 0;
  if (Auto ==0)
     {
     M_Ouv = 0 ; M_Fer = 0;   // Remise  zero  en Manuel
     }
  else
      {
      M_Ouv = 1; // Mmoire ouverture
      M_Fer = 0;
      }
  }
}

/* *******************Sous-Programme Fermeture******************************/
void SP_Fermeture (void)
{
if (M_Fer == 0)
  {
  C_Fer = 1;
  Delay10(Temps);// Commande de fermeture en fonction du volet
  C_Fer = 0;
  if (Auto ==0)
     {
     M_Ouv = 0 ; M_Fer = 0;   // Remise  zero  en Manuel
     }
  else
      {
      M_Fer = 1; // Mmoire Fermeture
      M_Ouv = 0;
      }
  }
}

/*--------------------------------Calcul Tempo ----------------*/
void Cal_Tempo (void)
/* calcul de la tempo en fonction des 2 dip switch */
{
if (b1T ==0 & b0T ==0) Temps =1500;   // 15 secondes
if (b1T ==0 & b0T ==1) Temps =2550;   // 25.5 secondes
if (b1T ==1 & b0T ==0) Temps =2900;   // 29 secondes
if (b1T ==1 & b0T ==1) Temps =4500;   // 45 secondes
} // Fin de calcul tempo
/*-------------------------------- Delay10 --------------------*/
void delay10( unsigned long int n)
/*
  Delays a multiple of 10 milliseconds using the TMR0 timer
  Clock : 4 MHz   => period T = 0.25 microseconds
  1 IS = 1 Instruction Cycle = 1 microsecond
  error: 0.16 percent
*/
{
    unsigned char i;
    i = 39;
    do  {
        //clrwdt();  // only if watchdog enabled
        //i = TMR0 + 39; /* 256 microsec * 39 = 10 ms */
        TMR0 = 0;
        while ( i != TMR0 && Auto==1);  // Manu on avorte la tempo
    } while ( --n > 0 && Auto ==1);  // Manu on avorte la tempo
}

/*----------------------Conversion analogique -------------------*/
void CAN (void){ // fonction de conversion qui renvoie une valeur int (16 bits)
  ADCON0.ADON = 1;
//unsigned int resultat; // variable locale pour rsultat de la conversion
Penombre = 0; // initialisation de resultat = 0000 0000 0000 0000b
ADCON0.GO=1;  //lancement de la conversion
while (ADCON0.GO == 1); // ne fait rien tant que GO_DONE=1 cd attente de la fin de la conversion par passage  0 de GO_DONE

Penombre = ADRESH; // met les 2 bits de poids fort dans rsultat  - les 2 bits sont mis en position 0 et 1
                   // si ADRESH =00000011b alors resultat= 0000 0000 00000011b

Penombre =(Penombre <<8); // dcale de 8 bits vers la gauche = positionneles 2 bits en 8me et 9me positions
                        // rsultat vaut alors 0000 0011 0000 0000b

Penombre = Penombre + ADRESL; // ajoute les 8 bits de poids faible
//return resultat;
}
/* =====================  Fin du programme  ===================================*/